<?php

namespace App\Handlers\Activities;

use App\Models\CardRidingUseLog;
use App\Models\CardRidingUserBags;
use App\Models\Order;
use App\Repositories\CardRidingUseLogRepository;
use App\Repositories\CardRidingUserBagsRepository;
use App\Repositories\OrderRepository;

/**
 * 骑行卡优惠
 * Class RidingCardActivityHandler
 * @package App\Handlers\Activities
 */
class RidingCardActivityHandler
{

    protected $cardRidingUserBagsRepository = null;
    protected $cardRidingUseLogRepository = null;
    protected $orderRepository = null;

    public function __construct(CardRidingUserBagsRepository $cardRidingUserBagsRepository, CardRidingUseLogRepository $cardRidingUseLogRepository, OrderRepository $orderRepository)
    {
        $this->cardRidingUserBagsRepository = $cardRidingUserBagsRepository;
        $this->cardRidingUseLogRepository = $cardRidingUseLogRepository;
        $this->orderRepository = $orderRepository;
    }


    public function main(&$order)
    {

        if ($order->time_money <= 0) return false;

        //已优惠不能在同时享受
        if ((int)$order->preferential_type !== Order::PREFERENTIAL_NO) return false;

        $cardRiding = $this->cardRidingUserBagsRepository->isExist($order->user_id, $order->start_use_bike_time);
        if (empty($cardRiding)) return false;

        $cardLogs = $this->cardRidingUseLogRepository->isUseThisCardByOrderId($order->id);
        if ($cardLogs) return false;

        $discount_time_money = $cardRiding->deduction_money;
        if (bccomp($order->time_money, $cardRiding->deduction_money, 2) < 0) {
            $discount_time_money = $order->time_money;
        }

        //调整订单
        $order->preferential_money = $discount_time_money;
        $order->card_preferential_money = $discount_time_money;
        $order->order_money = bcsub($order->order_money, $order->preferential_money, 2);
        $order->pay_money = $order->order_money;
        $order->preferential_type = Order::PREFERENTIAL_CARD_RIDING; //优惠方式


        if($order->pay_money == 0){
            $order->pay_type = Order::PAY_TYPE_ACCOUNT;
        }

        if ((int)$cardRiding->is_limit_times === CardRidingUserBags::LIMIT_TIMES_YES) {
            $cardRiding->can_ridding_times -= 1;
            if ($cardRiding->can_ridding_times <= 0) {
                // 如果没次数了 直接使其失效
                $cardRiding->status = CardRidingUserBags::STATUS_NO;
            }
            $cardRiding->save();
        }

        CardRidingUseLog::logs($cardRiding->id, $order->id, $order->preferential_money);
    }
}
