<?php
/**
 * Created by PhpStorm.
 * User: Mead
 * Date: 2019/8/5
 * Time: 2:44 PM
 */

namespace App\Transformers;

use App\Models\User;
use App\Repositories\OrderRepository;
use App\Repositories\RentOrderRepository;
use League\Fractal\TransformerAbstract;

class UserStatusTransformer extends TransformerAbstract
{
    protected $orderRepository;
    protected $rentOrderRepository;

    public function __construct(OrderRepository $orderRepository, RentOrderRepository $rentOrderRepository)
    {
        $this->orderRepository = $orderRepository;
        $this->rentOrderRepository = $rentOrderRepository;
    }

    public function transform(User $user)
    {
        return [
            'is_card_certified' => $user->is_card_certified,
            'is_deposit' => $user->is_deposit,
            'is_match_ride_age' => $user->is_match_ride_age,
            'is_bind_mobile' => $user->is_bind_mobile,
            'status' => $user->status,
            'is_register' => $user->is_register,
            'is_ride_order' => $this->rideOrder($user),
            'is_pay_order' => $this->noPayOrder($user),
            'is_rent_order' => $this->rentOrder($user),
            'is_pay_rent_order' => $this->rentPayOrder($user),
        ];
    }

    protected function rentOrder($user)
    {
        $no = $this->rentOrderRepository->byUserIdCheckIsExistRentOrderWithNo($user->id);
        if (is_null($no)) return false;
        return $no;
    }

    public function rentPayOrder($user)
    {
        $no = $this->rentOrderRepository->byUserIdCheckIsExistNoPayOrderWithNo($user->id);
        if (is_null($no)) return false;
        return $no;
    }

    protected function rideOrder($user)
    {
        $no = $this->orderRepository->byUserIdCheckIsExistRideOrderWithNo($user->id);
        if (is_null($no)) return false;
        return $no;
    }

    protected function noPayOrder($user)
    {
        $no = $this->orderRepository->byUserIdCheckIsExistNoPayOrderWithNo($user->id);
        if (is_null($no)) return false;
        return $no;
    }

}
